<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Backend
 */

/**
 * Imports data from a DHL Easylog export file into a SilvercartShipmentDhl
 * object
 *
 * @package SilvercartShipmentDhl
 * @subpacke Backend
 * @author Sascha Koehler <skoehler@pixeltricks.de>,
 *         Sebastian Diel <sdiel@pixeltricks.de>
 * @since 14.10.2013
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartShipmentDhlCSVBulkLoader extends BulkLoader {
    
    /**
     * Delimiter character (Default: comma).
     *
     * @var string
     */
    public $delimiter = ';';
    
    /**
     * Enclosure character (Default: doublequote)
     *
     * @var string
     */
    public $enclosure = '"';
    
    /**
     * Identifies if the has a header row.
     * @var boolean
     */
    public $hasHeaderRow = false;

    /**
     * Contains the mapping for the DHL files.
     *
     * @var array
     */
    public $columnMap = array(
        'SITEMS_ID',
        'SITEMS_IDENTCODE',
        'SITEMS_STORNO',
        'SITEMS_LABELTYP',
        'ABS_CUSTOMERDECL',
        'AbsHausNr',
        'AbsName1',
        'AbsName2',
        'ORT',
        'AbsPlz',
        'AbsStrasse',
        'SITEMS_DATE',
        'SITEMS_ABS_ID',
        'EmpKundenNr',
        'VerfahrenTyp',
        'SITEMS_READY_TO_SEND',
        'CircleNumber',
        'EmpName1',
        'EmpName2',
        'SITEMS_EMPBEM',
        'SITEMS_LEITCODE',
        'SITEMS_ABSBEM',
        'SITEMS_ORDERNUMBER',
        'KSTS_BEZEICHNUNG',
        'SITEMS_GEWICHT',
        'SITEMS_ZLEISTUNG',
        'SOWNER_CLOSEDATE',
        'SITEMS_NACHNAHME',
        'SITEMS_WERT',
        'SITEMS_ENTGELD',
        'EmpLandName',
        'EmpPlz',
        'SITEMS_PRODUKTCODE',
        'EmpOrt',
        'EmpStrasse',
        'SITEMS_PRODUKTNAME',
        'LeistungenText',
        'NNWAEHRUNG',
        'SITEMS_KOLLICOUNTER',
        'SITEMS_ZUSATZBETRAGNACHNAHME',
        'SITEMS_GEWICHT_MC',
        'SITEMS_VORAUSVERF',
        'EmpHausNr',
        'WertWaehrung',
        'SITEMS_STORNO_REASON',
        'TVWERTWAEHRUNG',
        'SITEMS_ZOLLZUSATZ',
        'EmpLandCode',
        'NKREIS_TYP_CN',
        'SITEMS_AWB',
        'SITEMS_TVWERT',
        'SITEMS_E_EMAIL',
        'SITEMS_E_GEBDATUM',
        'SITEMS_E_AUSWEISNR',
        'SITEMS_E_AUSWEISART',
        'SITEMS_E_AUSWEISBEH',
        'SITEMS_E_MINDESTALTER',
        'SITEMS_NACHNAME',
        'SITEMS_VORNAME',
        'SITEMS_MA_STADTTEIL',
        'SITEMS_MA_ORT',
        'SITEMS_MA_STRASSE',
        'SITEMS_VERTRAG_ID',
        'SITEMS_IDP_CONF',
        'SITEMS_TAUSCHBOX',
        'SITEMS_ZLEISTUNG_RETOUR',
        'SITEMS_IDENTCODE_RETOUR',
        'TEILNAHME',
        'SITEMS_CREATOR',
        '',
        '',
        '',
        '',
        '',
    );
    
    /**
     * Process every record in the file
     *
     * @param string  $filepath Absolute path to the file we're importing (with UTF8 content)
     * @param boolean $preview  If true, we'll just output a summary of changes but not actually do anything
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 30.08.2011
     */
    protected function processAll($filepath, $preview = false) {
        $csv = new CSVParser($filepath, $this->delimiter, $this->enclosure);
        
        if ($this->columnMap) {
            $csv->provideHeaderRow($this->columnMap);
        }
        
        foreach ($csv as $row) {
            $this->processRecord($row, $this->columnMap, $results, $preview);
        }
    }
    
    /**
     * Process a single record
     *
     * @param array             $record    An map of the data, keyed by the header field defined in {@link self::$columnMap}
     * @param array             $columnMap The column map
     * @param BulkLoader_Result &$result   BulkLoader_Result (passed as reference)
     * @param boolean           $preview   Indicates wether data should be really saved
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 21.09.2012
     */
    protected function processRecord($record, $columnMap, &$result, $preview = false) {
        if (!empty($record['EmpKundenNr']) &&
            !empty($record['SITEMS_ORDERNUMBER'])) {
            
            if (substr($record['EmpKundenNr'], 0, 5) == 'CSTER') {
                $customerNumber = substr($record['EmpKundenNr'], 5);
            } else {
                $customerNumber = $record['EmpKundenNr'];
            }
            
            if (substr($record['SITEMS_ORDERNUMBER'], 0, 5) == 'ORDER') {
                $orderNumber = substr($record['SITEMS_ORDERNUMBER'], 5);
            } else {
                $orderNumber = $record['SITEMS_ORDERNUMBER'];
            }
            
            $silvercartShipmentDhlImport = DataObject::get_one(
                'SilvercartShipmentDhlImport',
                sprintf(
                    "SITEMS_ID = '%s'",
                    $record['SITEMS_ID']
                )
            );
            
            if (!$silvercartShipmentDhlImport) {
                $silvercartShipmentDhlImport = new SilvercartShipmentDhlImport();
            }
            
            $silvercartShipmentDhlImport->setField('SITEMS_ID',                 $record['SITEMS_ID']);
            $silvercartShipmentDhlImport->setField('SITEMS_IDENTCODE',          $record['SITEMS_IDENTCODE']);
            $silvercartShipmentDhlImport->setField('SITEMS_STORNO',             $record['SITEMS_STORNO']);
            $silvercartShipmentDhlImport->setField('SITEMS_DATE',               $record['SITEMS_DATE']);
            $silvercartShipmentDhlImport->setField('EmpKundenNr',               $customerNumber);
            $silvercartShipmentDhlImport->setField('SITEMS_LEITCODE',           $record['SITEMS_LEITCODE']);
            $silvercartShipmentDhlImport->setField('SITEMS_ORDERNUMBER',        $orderNumber);
            $silvercartShipmentDhlImport->setField('SOWNER_CLOSEDATE',          $record['SOWNER_CLOSEDATE']);
            $silvercartShipmentDhlImport->setField('SITEMS_KOLLICOUNTER',       $record['SITEMS_KOLLICOUNTER']);
            $silvercartShipmentDhlImport->setField('SITEMS_STORNO_REASON',      $record['SITEMS_STORNO_REASON']);

            if (array_key_exists('SITEMS_IDENTCODE_RETOUR', $record)) {
                $silvercartShipmentDhlImport->setField('SITEMS_IDENTCODE_RETOUR', $record['SITEMS_IDENTCODE_RETOUR']);
            }
            if (array_key_exists('TEILNAHME', $record)) {
                $silvercartShipmentDhlImport->setField('TEILNAHME', $record['TEILNAHME']);
            }

            $silvercartShipmentDhlImport->write();
            
            // Write tracking code to order object
            $orderObj = DataObject::get_one(
                'SilvercartOrder',
                sprintf(
                    "OrderNumber = '%s'",
                    $silvercartShipmentDhlImport->SITEMS_ORDERNUMBER
                )
            );
            
            if ($orderObj) {
                $shipmentDhlConfig          = DataObject::get_one('SilvercartShipmentDhl');
                $orderObj->trackingCodeNVE  = $silvercartShipmentDhlImport->SITEMS_IDENTCODE;
                if ($shipmentDhlConfig &&
                    $shipmentDhlConfig->OrderStatusAfterImportID > 0 &&
                    $shipmentDhlConfig->OrderStatusAfterImport() instanceof SilvercartOrderStatus) {
                    $orderObj->SilvercartOrderStatusID = $shipmentDhlConfig->OrderStatusAfterImportID;
                }
                $orderObj->write();
            }
        }
    }
}