<?php
/**
 * Exports all not yet exported orders in DHL format.
 *
 * This task should be called via sake on the command line:
 * "sake /SilvercartShipmentDhlExportTask"
 *
 * You can set the following parameters:
 * "force": generate the export regardless of the update interval setting
 *
 * Example with parameters:
 *
 *   Generate export regardless of the update interval setting:
 *   "sake /SilvercartShipmentDhlExportTask force
 *
 *
 * @package SilvercartShipmentDhl
 * @subpackage Tasks
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 05.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlExportTask extends ScheduledTask {
    
    /**
     * This method gets called from sake.
     * 
     * You can give the argument "force" to generate the export
     * regardless of the update interval setting.
     *
     * It starts the export of all due SilvercartShipmentDhl objects.
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @copyright 2011 pixeltricks GmbH
     * @since 05.08.2011
     */
    public function process() {
        $force = false;
        
        if (isset($_GET['args']) &&
            is_array($_GET['args'])) {
            
            foreach ($_GET['args'] as $argument) {
                if ($argument == 'force') {
                    $force = true;
                }
            }
        }
        
        $referenceCurrentTimeStamp  = time();
        
        $silvercartShipmentDhl = $this->getDueSilvercartShipmentDhl($referenceCurrentTimeStamp, $force);
        
        if ($silvercartShipmentDhl) {
            $tableListField = new SilvercartShipmentDhlOrderTableListField(
                'SilvercartShipmentDhlExportTaskListField',
                'SilvercartOrder',
                null,
                'hasBeenExportedToDhl = 0'
            );
            
            $tableListField->generateExportFileData();
        }
        
        return true;
    }
    
    /**
     * This method returns an array containing all SilvercartProductExporters
     * that should be run now (according to their update interval settings).
     *
     * @param Int     $referenceCurrentTimeStamp The timestamp to use as reference
     *                                           for calculating if an update is due.
     * @param Boolean $force                     Indicates wether the update interval should
     *                                           be ignored.
     * 
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @copyright 2011 pixeltricks GmbH
     * @since 05.08.2011
     */
    protected function getDueSilvercartShipmentDhl($referenceCurrentTimeStamp, $force = false) {
        $dueSilvercartShipmentDhl = null;
        $conversionTable               = array(
            'Minutes'           => 1,
            'Hours'             => 60,
            'Days'              => 1440,
            'Weeks'             => 10080,
            'Months'            => 312480,
            'Years'             => 3749760
        );
        
        $silvercartShipmentDhl = DataObject::get_one(
            'SilvercartShipmentDhl'
        );
        
        if ($force) {
            $dueSilvercartShipmentDhl = $silvercartShipmentDhl;
        } else {
            // Get difference in minutes from now to the last export
            $lastExportTimeStamp = strtotime($silvercartShipmentDhl->lastExportDateTime);
            $timeStampDifference = $referenceCurrentTimeStamp - $lastExportTimeStamp;
            $differenceInMinutes = $this->time_duration($timeStampDifference, 'm');

            // Get update interval in minutes
            $intervalLengthInMinutes = $silvercartShipmentDhl->updateInterval * $conversionTable[$silvercartShipmentDhl->updateIntervalPeriod];

            if ($differenceInMinutes > $intervalLengthInMinutes) {
                $dueSilvercartShipmentDhl = $silvercartShipmentDhl;
            }
        }
        
        return $dueSilvercartShipmentDhl;
    }
    
    /**
     * A function for making time periods readable
     * 
     * @param int    number of seconds elapsed
     * @param string which time periods to display
     * @param bool   whether to show zero time periods
     * 
     * @author Aidan Lister <aidan@php.net>
     * @since 05.08.2011
     * @version 2.0.1
     * @link http://aidanlister.com/2004/04/making-time-periods-readable/
     */
    protected function time_duration($seconds, $use = null, $zeros = false) {
        // Define time periods
        $periods = array (
            'years'     => 31556926,
            'Months'    => 2629743,
            'weeks'     => 604800,
            'days'      => 86400,
            'hours'     => 3600,
            'minutes'   => 60,
            'seconds'   => 1
            );

        // Break into periods
        $seconds = (float) $seconds;
        $segments = array();
        foreach ($periods as $period => $value) {
            if ($use && strpos($use, $period[0]) === false) {
                continue;
            }
            $count = floor($seconds / $value);
            if ($count == 0 && !$zeros) {
                continue;
            }
            $segments[strtolower($period)] = $count;
            $seconds = $seconds % $value;
        }

        // Build the string
        $string = array();
        foreach ($segments as $key => $value) {
            $segment_name = substr($key, 0, -1);
            $segment = $value . ' ' . $segment_name;
            if ($value != 1) {
                $segment .= 's';
            }
            $string[] = $segment;
        }

        return implode(', ', $string);
    }
}