<?php
/**
 * Imports all not yet imported DHL Easylog files into the shop.
 *
 * This task should be called via sake on the command line:
 * "sake /SilvercartShipmentDhlImportTask"
 *
 * Example:
 *
 *   Generate export regardless of the update interval setting:
 *   "sake /SilvercartShipmentDhlImportTask
 *
 *
 * @package SilvercartShipmentDhl
 * @subpackage Tasks
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 30.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlImportTask extends ScheduledTask {
    
    /**
     * Contains the path to the import directory.
     * 
     * @var string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 30.08.2011
     */
    protected $importDirectory;

    /**
     * Contains the configuration object.
     *
     * @var SilvercartShipmentDhl
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 30.08.2011
     */
    protected $silvercartShipmentDhl;
    
    /**
     * We set all needed information regard directories, objects etc here.
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 30.08.2011
     */
    public function __construct() {
        parent::__construct();
        
        $silvercartShipmentDhl = DataObject::get_one(
            'SilvercartShipmentDhl'
        );
        
        if ($silvercartShipmentDhl &&
            !empty($silvercartShipmentDhl->importDirectory) &&
            is_dir($silvercartShipmentDhl->importDirectory)) {

            $this->importDirectory = $silvercartShipmentDhl->importDirectory;
        } else {
            $this->importDirectory = Director::baseFolder().'/silvercart_shipment_dhl/import_files/';
        }
        
        if (substr($this->importDirectory, -1) != '/') {
            $this->importDirectory .= '/';
        }
    }
    
    /**
     * This method gets called from sake.
     * 
     * It looks inside the import directory for files from DHL Easylog and
     * tries to match those files.
     * After the files have been worked off they are moved to a backup
     * directory.
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @copyright 2011 pixeltricks GmbH
     * @since 30.08.2011
     */
    public function process() {
        $loader = new SilvercartShipmentDhlCSVBulkLoader('SilvercartShipmentDhl');
        $files = $this->getFilesForImport();

        foreach ($files as $file) {
            $result = $loader->load($file);
            unlink($file);
        }
        
        return true;
    }
    
    /**
     * Gets the configured import directory and looks for DHL files to import.
     * Returns an array with all found filepaths.
     * 
     * The files must have the extension 'txt' and must start with the string
     * 'ELSendEx'.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 30.08.2011
     */
    protected function getFilesForImport() {
        $filesToImport = array();
        $files         = scandir($this->importDirectory);
        
        foreach ($files as $file) {
            if (is_file($this->importDirectory.$file)) {
                $fileInfo = pathinfo($this->importDirectory.$file);
                
                if (array_key_exists('extension', $fileInfo) &&
                    strtolower($fileInfo['extension']) == 'txt') {
                    
                    if (substr($fileInfo['basename'], 0, strlen('ELSendEx')) === 'ELSendEx') {
                        $filesToImport[] = $this->importDirectory.$file;
                    }
                }
            }
        }
        
        return $filesToImport;
    }
}